package iotcloud;
import java.io.*;
import java.net.*;
import java.util.Arrays;
import javax.crypto.*;

class CloudComm {
	String baseurl;
	Cipher encryptcipher;
	Cipher decryptcipher;
	Mac mac;

	CloudComm() {
	}

	CloudComm(String _baseurl, Cipher _encrypt, Cipher _decrypt, Mac _mac) {
		this.baseurl=_baseurl;
		this.encryptcipher = _encrypt;
		this.decryptcipher = _decrypt;
		this.mac = _mac;
	}

	private URL buildRequest(boolean isput, long sequencenumber, long maxentries) throws IOException {
		String reqstring=isput?"req=putslot":"req=getslot";
		String urlstr=baseurl+"?"+reqstring+"&seq="+sequencenumber;
		if (maxentries != 0)
			urlstr += "&max="+maxentries;
		return new URL(urlstr);
	}

	public Slot[] putSlot(Slot slot, int max) {
		try {
			long sequencenumber=slot.getSequenceNumber();
			byte[] bytes=slot.encode(mac);

			URL url=buildRequest(true, sequencenumber, max);
			URLConnection con=url.openConnection();
			HttpURLConnection http = (HttpURLConnection) con;
			http.setRequestMethod("POST");
			http.setFixedLengthStreamingMode(bytes.length);
			http.setDoOutput(true);
			http.connect();
			OutputStream os=http.getOutputStream();
			os.write(bytes);
			System.out.println(http.getResponseMessage());

			InputStream is=http.getInputStream();
			DataInputStream dis=new DataInputStream(is);
			byte[] resptype=new byte[7];
			dis.readFully(resptype);
			if (Arrays.equals(resptype, "getslot".getBytes()))
				return processSlots(dis);
			else if (Arrays.equals(resptype, "putslot".getBytes()))
				return null;
			else
				throw new Error("Bad response to putslot");
		} catch (Exception e) {
			throw new Error("putSlot failed");
		}
	}

	public Slot[] getSlots(long sequencenumber) {
		try {
			URL url=buildRequest(false, sequencenumber, 0);
			URLConnection con=url.openConnection();
			HttpURLConnection http = (HttpURLConnection) con;
			http.setRequestMethod("POST");
			http.connect();
			System.out.println(http.getResponseMessage());
			InputStream is=http.getInputStream();

			DataInputStream dis=new DataInputStream(is);
			byte[] resptype=new byte[7];
			dis.readFully(resptype);
			if (!Arrays.equals(resptype, "getslot".getBytes()))
				throw new Error("Bad Response: "+new String(resptype));
			else
				return processSlots(dis);
		} catch (Exception e) {
			throw new Error("getSlots failed");
		}
	}

	Slot[] processSlots(DataInputStream dis) throws IOException {
		int numberofslots=dis.readInt();
		int[] sizesofslots=new int[numberofslots];
		Slot[] slots=new Slot[numberofslots];
		for(int i=0; i<numberofslots; i++)
			sizesofslots[i]=dis.readInt();

		for(int i=0; i<numberofslots; i++) {
			byte[] data=new byte[sizesofslots[i]];
			dis.readFully(data);
			slots[i]=Slot.decode(data, mac);
		}
		dis.close();
		return slots;
	}
}
